$(document).ready(function() {
    var applet = new PulfrichApplet();
    applet.start();
});

var PulfrichApplet = function PulfrichApplet(options) {
    var defaults = {
    };

    var STIMULI_MARGIN = 6;
    var STIMULI_HEIGHT_RATIO = 0.15;
    var STIMULI_HEIGHT_MIN_PX = 100;

    this.settings = null;
    this.wallTextures = null;
    this.width;
    this.height;

    this.shouldRun = false;
    this.frameIndex = 0;

    this.frequency = 1;
    this.offset = 0.000;
    this.brightness = 0.8;
    this.disparity = 10;
    this.glassType = 'redCyan';


    var that = this;

    this.init = function(options) {
        // settings
        this.settings = $.extend(defaults, options);

        this._initFilters();

        this._initResize();

        this._initUI();
    }

    this.start = function() {
        this.shouldRun = true;
        this._loop();
    }

    this.stop = function() {
        this.shouldRun = false;
    }

    this._initResize = function() {
        $(window).resize(function() {
            that._resize();
        });
        this._resize();
    }

    this._initUI = function() {
        var updateValues = function() {
            that.frequency = $(".frequency .slider").slider("value");
            $(".frequency .label").val(that.frequency);
            that.offset = $(".offset .slider").slider("value") / 1000;
            $(".offset .label").val(that.offset * 1000);
            that.brightness = $(".brightness .slider").slider("value") / 100;
            $(".brightness .label").val(that.brightness * 100);
            that.disparity = $(".disparity .slider").slider("value");
            $(".disparity .label").val(that.disparity);
            
            $("body").css("background-color" , getCssColorF(0.2 * that.brightness, 0.2 * that.brightness, 0.2 * that.brightness));
        };

        $(".frequency .slider").slider({
            orientation: "horizontal",
            range: "min",
            min: 0,
            max: 2,
            value: this.frequency,
            step: 0.01,
            slide: updateValues,
            change: updateValues
        });

        $(".offset .slider").slider({
            orientation: "horizontal",
            range: "min",
            min: -20,
            max: 20,
            value: this.offset * 1000,
            step: 0.5,
            slide: updateValues,
            change: updateValues
        });

        $(".brightness .slider").slider({
            orientation: "horizontal",
            range: "min",
            min: 0,
            max: 100,
            value: this.brightness * 100,
            step: 10,
            slide: updateValues,
            change: updateValues
        });

        $(".disparity .slider").slider({
            orientation: "horizontal",
            range: "min",
            min: -20,
            max: 40,
            value: this.disparity,
            step: 1,
            slide: updateValues,
            change: updateValues
        });

        $("#controls .label").change(function() {
            var value = parseFloat($(this).val());
            if (!isNaN(value)) {
                var slider = $(this).parents("tr:eq(0)").find(".slider");
                slider.slider("option", "value", value);
            }
            updateValues();
        });
        $("#controls .label").keydown(function(event) {
            var slider = $(this).parents("tr:eq(0)").find(".slider");
            var value = slider.slider("option", "value");
            var step = slider.slider("option", "step");

            var code = (event.keyCode ? event.keyCode : event.which);
            if (code == 38) {
                value += step; // Arrow Up
                event.preventDefault();
            } else if (code == 40) {
                value -= step; // Arrow Down
                event.preventDefault();
            }
            slider.slider("option", "value", value);
        });

        $("#controls > table").mousewheel(function(event, delta, deltaX, deltaY) {
            var focusedElement = $(':focus:eq(0)');
            var slider = focusedElement.parents("tr:eq(0)").find(".slider");
            if (slider.size() == 0) {
                return;
            }
            event.preventDefault();
            var value = slider.slider("option", "value");
            var step = slider.slider("option", "step");
            value += deltaY * step;
            slider.slider("option", "value", value);
        });

        $("#controls .glassType select").change(function() {
            that.glassType = $(this).val();
        });
        $("#controls .glassType select").change();

        updateValues();
    };

    this._initFilters = function() {

        this.wallTextures = {
            redCyan: [$("#wallTextureRed").get(0), $("#wallTextureCyan").get(0)],
            greenMagenta: [$("#wallTextureGreen").get(0), $("#wallTextureMagenta").get(0)],
            colorCode3D: [$("#wallTextureAmber").get(0), $("#wallTextureBlue").get(0)]
        };

        this.filterColors = {
            redCyan: [[1, 0, 0], [0, 1, 1]],
            greenMagenta: [[0, 1, 0], [1, 0, 1]],
            colorCode3D: [[1, 1, 0], [0, 0, 1]]
        }
    };

    this._loop = function() {
        if (!this.shouldRun) {
            return;
        }

        var activeCanvasIndex = this.frameIndex % 2;
        var passiveCanvasIndex = (this.frameIndex + 1) % 2;
        this.frameIndex++;

        var activeCanvas = $(".canvas").get(activeCanvasIndex);
        var passiveCanvas = $(".canvas").get(passiveCanvasIndex);
        var ctx = activeCanvas.getContext('2d');

        this._render(ctx);

        passiveCanvas.style.display = 'none';
        activeCanvas.style.display = 'block';

        setTimeout(function() {
            that._loop()
        }, 0);
    }

    this._render = function(ctx) {
        var time = this.getTime();

        // size
        var activeHeight = this.height * STIMULI_HEIGHT_RATIO;
        activeHeight = Math.max(activeHeight, STIMULI_HEIGHT_MIN_PX);
        var borderHeight = (this.height - activeHeight) / 2;

        // data
        var wallTextureLeft = this.wallTextures[this.glassType][0];
        var wallTextureRight = this.wallTextures[this.glassType][1];
        var filterLeft = this.filterColors[this.glassType][0];
        var filterRight = this.filterColors[this.glassType][1];
        var maxDisparity = parseFloat($(".disparity .slider").slider("option", "max"));
        var bgWidth = this.width - 2 * maxDisparity / 2;

        ctx.globalCompositeOperation = "source-over";
        ctx.fillStyle = getCssColorF(0.2 * this.brightness, 0.2 * this.brightness, 0.2 * this.brightness);
        ctx.fillRect(0, 0, this.width, this.height);
        ctx.fillStyle = "#000000";
        ctx.fillRect((maxDisparity - this.disparity) / 2, 0, bgWidth + this.disparity, borderHeight);
        ctx.fillRect((maxDisparity - this.disparity) / 2, this.height - borderHeight, bgWidth + this.disparity, borderHeight);

        // background
        ctx.globalAlpha = this.brightness;
        ctx.globalCompositeOperation = "lighter";
        drawImageRepeat(ctx, wallTextureLeft, (maxDisparity - this.disparity) / 2, 0, bgWidth, borderHeight);
        drawImageRepeat(ctx, wallTextureRight, (maxDisparity + this.disparity) / 2, 0, bgWidth, borderHeight);
        drawImageRepeat(ctx, wallTextureLeft, (maxDisparity - this.disparity) / 2, this.height - borderHeight, bgWidth, borderHeight);
        drawImageRepeat(ctx, wallTextureRight, (maxDisparity + this.disparity) / 2, this.height - borderHeight, bgWidth, borderHeight);
        ctx.globalAlpha = 1;
        ctx.globalCompositeOperation = "source-over";

        // stimula
        var positionRight = 0.5 + 0.4 * Math.sin(this.frequency * 3.1415 * time);
        var positionLeft = 0.5 + 0.4 * Math.sin(this.frequency * 3.1415 * (time + this.offset));
        var stimuliSize = activeHeight - 2 * STIMULI_MARGIN;
        var stimuliY = borderHeight + STIMULI_MARGIN;
        var stimuliXLeft = STIMULI_MARGIN + positionLeft * (this.width - 2 * STIMULI_MARGIN - stimuliSize);
        var stimuliXRight = STIMULI_MARGIN + positionRight * (this.width - 2 * STIMULI_MARGIN - stimuliSize) + this.disparity;

        var leftStartX = stimuliXLeft;
        var leftEndX = stimuliXLeft + stimuliSize;

        var rightStartX = stimuliXRight;
        var rightEndX = stimuliXRight + stimuliSize;


        // left		
        ctx.globalCompositeOperation = "lighter";
        ctx.fillStyle = getCssColorF(filterLeft[0] * this.brightness, filterLeft[1] * this.brightness, filterLeft[2] * this.brightness);
        ctx.fillRect(leftStartX, stimuliY, leftEndX - leftStartX, stimuliSize);

        // right
        ctx.fillStyle = getCssColorF(filterRight[0] * this.brightness, filterRight[1] * this.brightness, filterRight[2] * this.brightness);
        ctx.fillRect(rightStartX, stimuliY, rightEndX - rightStartX, stimuliSize);
    };

    this._resize = function() {
        var screenSize = getScreenSize();
        this.width = screenSize[0] - 100;
        this.height = screenSize[1] - 240;
        $(".canvas").attr("width", this.width);
        $(".canvas").attr("height", this.height);
    }

    this.getTime = function() {
        var d = new Date();
        return d.getTime() / 1000.0;
    }

    var drawImageRepeat = function(ctx, image, x, y, width, height) {
        var imageWidth = image.width;
        var imageHeight = image.height;
        if (imageWidth <= 0 || imageHeight <= 0) {
            return;
        }

        for (var offsetY = 0; offsetY < height; offsetY += imageHeight) {
            var blockHeight = Math.min(height - offsetY, imageHeight);
            for (var offsetX = 0; offsetX < width; offsetX += imageWidth) {
                var blockWidth = Math.min(width - offsetX, imageWidth);
                ctx.drawImage(image, 0, 0, blockWidth, blockHeight, x + offsetX, y + offsetY, blockWidth, blockHeight);
            }
        }
    };

    var getScreenSize = function() {
        var winW = 630, winH = 460;
        if (document.body && document.body.offsetWidth) {
            winW = document.body.offsetWidth;
            winH = document.body.offsetHeight;
        }
        if (document.compatMode == 'CSS1Compat' &&
                document.documentElement &&
                document.documentElement.offsetWidth) {
            winW = document.documentElement.offsetWidth;
            winH = document.documentElement.offsetHeight;
        }
        if (window.innerWidth && window.innerHeight) {
            winW = window.innerWidth;
            winH = window.innerHeight;
        }

        return [winW, winH];
    }

    var getCssColorF = function(r, g, b) {
        return 'rgb(' + Math.round(r * 255) + ',' + Math.round(g * 255) + ',' + Math.round(b * 255) + ')';
    };

    this.init(options);
}